@@ -41,6 +41,20 @@ CREDIT = u"credit"
 PRIOR_AUTH_CAPTURE = u"prior_auth_capture"
 VOID = u"void"
 
+class AuthorizeSystemError(Exception):
+    """
+    I'm a serious kind of exception and I'm raised when something
+    went really bad at a lower level than the application level, like
+    when Authorize is down or when they return an unparseable response
+    """
+    def __init__(self, *args):
+        self.args = args
+    def __str__(self):
+        return "Exception: %s caused by %s" % self.args
+    def __repr__(self):
+        # Here we are printing a tuple, the , at the end is _required_
+        return "AuthorizeSystemError%s" % (self.args,)
+
 c = re.compile(r'([A-Z]+[a-z_]+)')
 
 def convert(arg):
@@ -72,6 +86,15 @@ def convert(arg):
         raise Exception("'%s' not unicode: can only accept unicode strings" % (arg,))
     raise Exception("Cannot convert %s of type %s" % (arg, type(arg)))
 
+def utf8convert(arg):
+    """
+    Further extend L{convert} to return UTF-8 strings instead of unicode.
+    """
+    value = convert(arg)
+    if isinstance(value, unicode):
+        return value.encode('utf-8')
+    return value
+
 class XMLBuilder(object):
     """
     XMLBuilder tries to be slightly clever in order to be easier for
@@ -158,24 +181,39 @@ def parse_node(node):
             new[tag].append(child)
     return new
 
-def to_dict(s, error_codes, do_raise=True):
+def to_dict(s, error_codes, do_raise=True, delimiter=u',', encapsulator=u'', uniform=False):
     """
     Return a dict_accessor representation of the given string, if raise_
     is True an exception is raised when an error code is present.
     """
-    t = fromstring(s)
+    try:
+        t = fromstring(s)
+    except SyntaxError, e:
+        raise AuthorizeSystemError(e, s)
+
     parsed = dict_accessor(parse_node(t)) # discard the root node which is useless
     try:
         if isinstance(parsed.messages.message, list): # there's more than a child
             return parsed
         code = parsed.messages.message.code.text_
+        if uniform:
+            parsed.messages.message = [parsed.messages.message]
     except KeyError:
         return parsed
     if code in error_codes:
         if do_raise:
             raise error_codes[code]
+
+    dr = None
     if parsed.get('direct_response') is not None:
-        parsed.direct_response = parse_direct_response(parsed.direct_response.text_)
+        dr = parsed.direct_response.text_
+    elif parsed.get('validation_direct_response') is not None:
+        dr = parsed.validation_direct_response.text_
+
+    if dr is not None:
+        parsed.direct_response = parse_direct_response(dr,
+                                                       delimiter,
+                                                       encapsulator)
     return parsed
 
 
@@ -188,14 +226,14 @@ m = ['code', 'subcode', 'reason_code', 'reason_text', 'auth_code',
      'freight', 'tax_exempt', 'po_number', 'md5_hash', 'ccv',
      'holder_verification']
 
-def parse_direct_response(s):
+def parse_direct_response(s, delimiter=u',', encapsulator=u''):
     """
     Very simple format but made of many fields, the most complex ones
     have the following meanings:
-    
-        code: 
+
+        code:
             see L{responses.aim_codes} for all the codes
-            
+
         avs:
             see L{responses.avs_codes} for all the codes
 
@@ -208,9 +246,9 @@ def parse_direct_response(s):
             CREDIT
             PRIOR_AUTH_CAPTURE
             VOID
-        
+
         tax_exempt: true, false, T, F, YES, NO, Y, N, 1, 0
-        
+
         ccv:
             see L{responses.ccv_codes} for all the codes
 
@@ -219,12 +257,27 @@ def parse_direct_response(s):
     """
     if not isinstance(s, unicode):
         s = s.decode('utf-8', 'replace')
-        
-    v = s.split(u',')
-    if not len(v) >= len(m):
-        v = s.split(u'|')
 
-    d = dict_accessor(dict(zip(m, v)))
+    # being <e> the encapsulator and <d> the delimiter
+    # this is the format of the direct response:
+    # <e>field<e><d><e>field<e><d><e>field<e>
+    #
+    # Here's a regexp that would parse this:
+    #    "\<e>([^\<d>\<e>]*)\<e>\<d>?"
+    # But it has a problem when <e> is '' and I don't
+    # have the will to do the much harder one that actually
+    # does it well... So let's just split and strip.
+    e = encapsulator
+    d = delimiter
+
+    v = s.split(e+d+e)
+    v[0] = v[0].lstrip(e)
+    v[-1] = v[-1].rstrip(e)
+
+    if not len(v) >= len(m):
+        d = dict_accessor({'error': "Couldn't parse the direct response"})
+    else:
+        d = dict_accessor(dict(zip(m, v)))
     d.original = s
     return d
 
@@ -254,7 +307,7 @@ def _address(pre='', kw={}, *extra):
         x.state(kw.get(pre+'state')), # optional
         x.zip(kw.get(pre+'zip')), # optional
         x.country(kw.get(pre+'country')) # optional
-        
+
     ] + list(extra)
 
 def address(pre='', **kw):
@@ -294,6 +347,10 @@ def billTo(**kw):
     )
 
 def arbBillTo(**kw):
+    # This is just to be sure that they were passed.
+    # as the spec requires
+    kw['bill_first_name']
+    kw['bill_last_name']
     return x.billTo(
         *_address('bill_', kw)
     )
@@ -368,17 +425,25 @@ def transaction(**kw):
         x.customerProfileId(kw['customer_profile_id']),
         x.customerPaymentProfileId(kw['customer_payment_profile_id']),
         x.customerAddressId(kw.get('customer_address_id')),
-        x.order(
-            x.invoiceNumber(kw.get('invoice_number')),
-            x.description(kw.get('description')),
-            x.purchaseOrderNumber(kw.get('purchase_order_number'))
-        ),
-        x.taxExempt(kw.get('tax_exempt', False)),
-        x.recurringBilling(kw.get('recurring', False)),
-        x.cardCode(kw.get('ccv'))
     ]
-    
+
     ptype = kw.get('profile_type', AUTH_ONLY)
+    if ptype in (AUTH_ONLY, CAPTURE_ONLY, AUTH_CAPTURE, CREDIT):
+        content += [
+            x.order(
+                x.invoiceNumber(kw.get('invoice_number')),
+                x.description(kw.get('description')),
+                x.purchaseOrderNumber(kw.get('purchase_order_number'))
+            )
+        ]
+    if ptype in (AUTH_ONLY, CAPTURE_ONLY, AUTH_CAPTURE):
+        content += [
+            x.taxExempt(kw.get('tax_exempt', False)),
+            x.recurringBilling(kw.get('recurring', False)),
+            x.cardCode(kw.get('ccv'))
+        ]
+
+
     if ptype == AUTH_ONLY:
         profile_type = x.profileTransAuthOnly(
             *content
@@ -391,8 +456,26 @@ def transaction(**kw):
         profile_type = x.profileTransAuthCapture(
             *content
         )
+    elif ptype == PRIOR_AUTH_CAPTURE:
+        profile_type = x.profileTransPriorAuthCapture(
+            *(content + [x.transId(kw['trans_id'])])
+        )
+    # NOTE: It is possible to issue a refund without the customerProfileId and
+    # the customerPaymentProfileId being supplied. However, this is not
+    # currently supported, and requires sending the masked credit card number.
+    elif ptype == CREDIT:
+        profile_type = x.profileTransRefund(
+            *(content + [x.transId(kw['trans_id'])])
+        )
+    elif ptype == VOID:
+        profile_type = x.profileTransVoid(
+            *(content + [x.transId(kw['trans_id'])])
+        )
+    else:
+        raise Exception("Unsupported profile type: %r" % (ptype,))
+
     return x.transaction(profile_type)
-    
+
 def paymentProfiles(**kw):
     return x.paymentProfiles(
         x.customerType(kw.get('customer_type')), # optional: individual, business
@@ -421,7 +504,7 @@ def profile(**kw):
         x.description(kw.get('description')),
         x.email(kw.get('email'))
     ]
-    
+
     payment_profiles = kw.get('payment_profiles', None)
     if payment_profiles is not None:
         content = content + list(
@@ -444,14 +527,14 @@ def subscription(**kw):
         x.name(kw.get('subscription_name')),
         x.paymentSchedule(
             x.interval(
-                x.length(kw['interval_length']), # up to 3 digits, 1-12 for months, 7-365 days
-                x.unit(kw['interval_unit']) # days or months
+                x.length(kw.get('interval_length')), # up to 3 digits, 1-12 for months, 7-365 days
+                x.unit(kw.get('interval_unit')) # days or months
             ),
-            x.startDate(kw['start_date']), # YYYY-MM-DD
+            x.startDate(kw.get('start_date')), # YYYY-MM-DD
             x.totalOccurrences(kw.get('total_occurrences', 9999)),
             x.trialOccurrences(trial_occurrences)
         ),
-        x.amount(kw['amount']),
+        x.amount(kw.get('amount')),
         x.trialAmount(trial_amount),
         payment(**kw),
         x.order(
